/*
 ******************************* Creazione file *******************************
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago             	Data di creazione : 23/12/98
 * Codice   :
 *
 * Definition of Exported Function 
 *
 ******************************* Revisione file *******************************
 *-------------------------------------|--------------------------------------*
 * Revisore : 								Data di revisione :
 * Versione :
 *
 * Note:
 *
 */

#include "stdafx.h"

/******************* Prototipi delle funzioni di servizio *********************
*--------------------------------------|--------------------------------------*/

/****************************** Variabili locali ******************************
*--------------------------------------|--------------------------------------*/

/***************************** Variabili globali ******************************
*--------------------------------------|--------------------------------------*/

HHOOK				hMouseHook		=	NULL;	//	Instance handle for Mouse Hook
TMSGQueue*			MsgQueueList	=	NULL;	//	Queues list
HINSTANCE			hInstance		=	NULL;	//	Instance handle for	DLL
TCriticalSection	CS;

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 22/12/1998
 * Codice   :
 *
 * Note:	Filter function for the WH_MOUSE
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

extern "C" LRESULT CALLBACK MouseProc (int nCode, WPARAM wParam, LPARAM lParam )
{
	LPMOUSEHOOKSTRUCT MouseHookParam;

	if ( nCode >= 0 ) 
	{
		switch(nCode)
		{
			case	HC_NOREMOVE:
				break;
			case	HC_ACTION:
			{	
				if(NULL!=(MouseHookParam = (MOUSEHOOKSTRUCT *) lParam))
				{
					TCriticalSection::TLock lock(CS);
					
					TMSGQueue*	queue	=	MsgQueueList;
			
					while(queue)
					{
						if(queue->IsValidWindow(MouseHookParam->hwnd))
						{
							if(queue->IsRunning() && !queue->IsFull() && queue->IsValidMessage(wParam))
							{
								MSG	msg;
								POINT	pt		=	MouseHookParam->pt;

								if(::ScreenToClient(MouseHookParam->hwnd, &pt))
									msg.lParam	=	MAKELPARAM( pt.x, pt.y);
								else
									msg.lParam	=	0;

								msg.hwnd		=	MouseHookParam->hwnd;
								msg.message		=	wParam;
								msg.wParam		=	MouseHookParam->wHitTestCode;
								msg.time		=	::GetTickCount();
								msg.pt			=	MouseHookParam->pt;
								queue->Insert(msg);
							}
						}
							
						queue=queue->next;
					}

					//	Return 0 to prevent recursion
					
					return LRESULT(0);
				}
				break;
			}
		}
   }

   //
   // We looked at the message ... sort of processed it but since we are
   // looking we will pass all messages on to CallNextHookEx.
   //
   return( CallNextHookEx(hMouseHook, nCode, wParam, lParam) );
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 22/12/1998
 * Codice   :
 *
 * Note:	DLL Entry Point
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */


BOOL APIENTRY DllMain( HINSTANCE hModule, DWORD  dw_reason_for_call, LPVOID lpReserved )
{
	hInstance=hModule;

    switch (dw_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		{
			if(hMouseHook!=NULL)
				break;

			hMouseHook=SetWindowsHookEx(	WH_MOUSE,       // type of hook to install
											MouseProc,		// address of hook procedure
											hInstance,		// handle to application instance
											NULL			// identity of thread to install hook for); 
											);
			break;
		}

		case DLL_THREAD_ATTACH:
			break;

		case DLL_THREAD_DETACH:
			break;

		case DLL_PROCESS_DETACH:
		{
			if(hMouseHook!=NULL)
				UnhookWindowsHookEx(hMouseHook);
			
			hMouseHook	=	NULL;
			
			TCriticalSection::TLock lock(CS);
			
			TMSGQueue*	queue	=	MsgQueueList;
	
			while(queue)
			{
				MsgQueueList	=	queue->next;
				
				delete	queue;

				queue	=	MsgQueueList;
			}

			break;
		}
    }

    return TRUE;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 22/12/1998
 * Codice   :
 *
 * Note:	Get the MessageQueue associated to a Window
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

BVMOUSEHOOK_API HANDLE	BVGetMouseQueue(HWND hWnd)
{
    TCriticalSection::TLock lock(CS);

	TMSGQueue*	queue	=	MsgQueueList;
	
	while(queue)
	{
		if(queue->IsValidWindow(hWnd))
			return queue;

		queue=queue->next;
	}

	return NULL;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 22/12/1998
 * Codice   :
 *
 * Note:	Check if the handle point to a valid Queue
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */


BVMOUSEHOOK_API BOOL	BVIsValidQueue(HANDLE hMsgQueue)
{
	if(hMsgQueue==NULL || MsgQueueList==NULL)
		return FALSE;

    TCriticalSection::TLock lock(CS);
	
	TMSGQueue*	queue	=	MsgQueueList;
	
	while(queue)
	{
		if(queue==(TMSGQueue*)hMsgQueue)
			return TRUE;
		
		queue=queue->next;
	}

	return FALSE;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 22/12/1998
 * Codice   :
 *
 * Note:	Install mouse hook
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */


BVMOUSEHOOK_API HANDLE BVInstallMouseHook(HWND hWnd, DWORD	dw_MinMsg,	DWORD	dw_MaxMsg, DWORD	dw_QueueSize, BOOL	b_start)
{
	if(!IsWindow(hWnd))
		return NULL;
	   
	TMSGQueue*	MsgQueue	=	new	TMSGQueue(hWnd, dw_MinMsg, dw_MaxMsg, dw_QueueSize, b_start);
	
	if(MsgQueue==NULL)
		return NULL;

	TCriticalSection::TLock lock(CS);

	if(MsgQueueList)
	{
		TMSGQueue*	queue	=	MsgQueueList;
		
		while(queue->next)
			queue=queue->next;
	
		queue->next	=	MsgQueue;
	}
	else
		MsgQueueList	=	MsgQueue;

	return (HANDLE)MsgQueue;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 22/12/1998
 * Codice   :
 *
 * Note:	Remove a previous installed mouse Hook
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */


BVMOUSEHOOK_API BOOL BVRemoveMouseHook(HANDLE hMsgQueue)
{
	if(!BVIsValidQueue(hMsgQueue))
		return FALSE;

    TCriticalSection::TLock lock(CS);
	
	TMSGQueue*	MsgQueue	=	(TMSGQueue*) hMsgQueue;

	if(MsgQueue==NULL)
		return	FALSE;

	if(MsgQueue!=MsgQueueList)
	{
		TMSGQueue*	queue	=	MsgQueueList;
		
		while(queue->next != MsgQueue)
			queue=queue->next;

		queue->next = queue->next->next;
	}
	else
	{
		MsgQueueList	=	MsgQueueList->next;
	}

	delete	MsgQueue;
	
	return	TRUE;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 22/12/1998
 * Codice   :
 *
 * Note:	Return if available a Mouse Event
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

BVMOUSEHOOK_API BOOL BVGetMouseEvent(HANDLE hMsgQueue, LPMSG lpMsg, BOOL b_Remove)
{
	if(!BVIsValidQueue(hMsgQueue))
		return FALSE;

	TMSGQueue*	MsgQueue	=	(TMSGQueue*) hMsgQueue;

	if(MsgQueue==NULL)
		return	FALSE;

    TCriticalSection::TLock lock(CS);

	if(MsgQueue->GetCount())
	{
		return	MsgQueue->Remove(*lpMsg, b_Remove);
	}

	return FALSE;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 22/12/1998
 * Codice   :
 *
 * Note:	Flush a mouse Queue 
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */


BVMOUSEHOOK_API BOOL	BVFlushMouseQueue(HANDLE hMsgQueue)
{
	if(!BVIsValidQueue(hMsgQueue))
		return FALSE;

    TCriticalSection::TLock lock(CS);

	TMSGQueue*	MsgQueue	=	(TMSGQueue*) hMsgQueue;

	if(MsgQueue==NULL)
		return	FALSE;

	MsgQueue->Flush();

	return TRUE;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 22/12/1998
 * Codice   :
 *
 * Note:	Enable Mouse Queue Capture
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

BVMOUSEHOOK_API BOOL	BVEnableMouseCapture(HANDLE hMsgQueue, BOOL b_enable)
{
	if(!BVIsValidQueue(hMsgQueue))
		return FALSE;

    TCriticalSection::TLock lock(CS);

	TMSGQueue*	MsgQueue	=	(TMSGQueue*) hMsgQueue;

	if(MsgQueue==NULL)
		return	FALSE;

	MsgQueue->b_running	= b_enable;

	return TRUE;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 22/12/1998
 * Codice   :
 *
 * Note:	Return TRUE if capturing queue is active
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

BVMOUSEHOOK_API BOOL	BVIsMouseCaptureEnabled(HANDLE hMsgQueue)
{
	if(!BVIsValidQueue(hMsgQueue))
		return FALSE;

    TCriticalSection::TLock lock(CS);

	TMSGQueue*	MsgQueue	=	(TMSGQueue*) hMsgQueue;

	if(MsgQueue==NULL)
		return	FALSE;

	return	MsgQueue->b_running;
}

